#case
// LF18Switch.c
// Push-Button Power Switch Demo
// Device:        Microchip PIC12LF1822/1840 and PIC16LF182x family
// Compiler:      Custom Computer Services, Inc. PCWH 4.135
// Programmer:    Microchip PICkit2
//
//              + VDD - 1   8 - VSS -
//                RA5 - 2   7 - RA0 -
//    On/Off PB   RA4 - 3   6 - RA1 - optional pFET, active low
//        MCLR -> RA3 - 4   5 - RA2 - LED, Relay, etc. active high
//
// 2014.05.28 - Jurgen G Schmidt

#include <12lf1840.h>
#FUSES INTRC_IO      //Internal RC Osc, no output
#FUSES NOWDT         //No Watch Dog Timer
#FUSES NOPUT         //No Power Up Timer
#FUSES NOMCLR        //Use RA3 as input
#FUSES NOBROWNOUT    //No brownout reset
#fuses NOCPD, NOLVP 
#fuses NOFCMEN       // saves .05 uA power in sleep

#use delay(internal=8M)       // no need to run at full speed

//------------------------------------- HARDWARE -------------------------------
#define  LED         PIN_A2   // Flashlight LED
#define  PB          PIN_A4   // pushbutton
#define  PFET        PIN_A1   // active low

//------------------------------------- Function Prototypes --------------------
void HWSetup( void );         // hardware setup after power-on / reset
void HWSleep( void );         // prepare hardware for low power sleep

//------------------------------------- main() ---------------------------------
void main()
{
  unsigned int16 dbCounter=0;           // debounce counter
   
  HWSetup();
  output_high( LED );                   // turn on system     
  output_low( PFET );

  //----------------------------------- BIG LOOP -------------------------------
  while( TRUE ) {                         
 
    if( input(PIN_A4) == 1 )            // check for button press
    {
      dbCounter++;
    } else {
      dbCounter = 0;
    }
    if( dbCounter > 20 ) break;         // break out of loop if held 2 seconds
      
    //--------------------------------------
    //-- do other embedded system stuff here
    //--------------------------------------
   
    delay_ms( 100 );                    // slow down for demo
      
  }//end of big loop
  //----------------------------------------------------------------------------
   
   delay_ms( 1000 );                    // give us time to release button
   HWSleep();                           // go back to sleep
}//end of main()

//------------------------------------- HWSetup() ------------------------------
void HWSetup( void )
{  
  // Check why we got started. Don't turn on if we just put in new batteries
  if( restart_cause() == NORMAL_POWER_UP ) HWSleep();
   
  setup_adc_ports( NO_ANALOGS );        // ALWAYS, Always, always do this !!!
   
  //------ Do other setups for fancier systems - this works for most of the 
  //------ Microchip XLP processors...
   
}//-- end of HWSetup()

//------------------------------------- HWSleep() ------------------------------
void HWSleep( void )
{
  //-- power down what we can...   
  setup_adc_ports( NO_ANALOGS);
  setup_vref( VREF_OFF );               // uses 16uA when on (by default in some)
  setup_timer_1( T1_DISABLED );         // turn off timers
  setup_timer_2( T2_DISABLED );
  output_a( 0x00 );                     // turn off everything
  output_high( PFET );                  // set high to turn off pFET
  enable_interrupts( INT_RA4_L2H );     // turn on interrupt on power button
  sleep();                              // go to sleep
    
  // Wakeup from sleep via button press resumes here...
  clear_interrupt( INT_RA4_L2H );       // clean up interrupt from sleep
  disable_interrupts( INT_RA4_L2H );
  reset_cpu();                          // do a full reset when we wake up
}//-- end of HWSleep()

//------------------------------------------------------------------------------
//------ end of LF18Switch.c ------
